// Reloader utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

import java.net.URL;

import com.myarch.reloader.*;

/**
 * This class demonstates different ways of reloading a class or a group of classes
 * using <code>Reloader</code> utility.
 * <p>
 * The idea of this program and sce code is borrowed from the JDC tech tip
 * <a href="http://developer.java.sun.com/developer/TechTips/2000/tt1027.html">
 * "CLASS LOADERS AS A NAMESPACE MECHANISM"</a>.
 * <p>Each example runs in turn, to repeat the example press R, to proceed to the next example
 * press Enter. Repeating the example means reloading the class.
 * For the example with the reloading from the custom path (non-JVM classpath), the
 * path can be specified as a parameter.
 * <p>See <code>TestConstants</code> for class names.
 *
 *
 * @author  Alexander Ananiev
 */
public class ReloaderSamples implements TestConstants {
  private static String customPath = CUSTOM_PATH;


  /**
   *  Runs all examples.
   */
  public void run() throws Exception {
    // turn the logging on. Alternatively, reloader.log system property can be set to "yes"
    ReloadLogger.setLoggingOn();
    // example 1. Only the main class is reloaded
    do {
      System.out.println("Example #1 -- Reloading the TestClass only -- TestClass2 won't be reloaded.");
      reloadMainClass();
    }
    while( readCommand() );

    // example 2. Package is reloaded
    do {
      System.out.println("Example #2 -- Reloading all classes in the package: "+TEST_PACKAGE);
      reloadPackage();
    }
    while( readCommand() );

    // example 3. Specified classes are reloaded
    do {
      System.out.println("Example #3 -- -- Reloading selected classes (TestClass and TestClass2).");
      reloadSelectedClasses();
    }
    while( readCommand() );

    // example 4. Classes other than specified (exceptions) are reloaded
    do {
      System.out.println("Example #4 -- Reloading all classes other than the specified (TestIface)"+
      "\n-- Note that java.lang classes are never reloaded, so no need to list them explicitly.");
      reloadExceptionClasses();
    }
    while( readCommand() );

    // example 5. Reloading all classes from the custom path (not on the classpath)
    do {
      System.out.println("Example #5 -- Reloading all classes in the non-JVM path. Classes not on the path are not reloaded. "+
        "\nPath: "+customPath);
      reloadCustomPath();
    }
    while( readCommand() );

    System.out.println("End of reloader examples");

  }

  /**
   * Reloads the test class, only one class is reloaded.
   */
  public void reloadMainClass() throws Exception {
    Reloader reloader = new Reloader();
    invokePrint( reloader, TEST_CLASS );
  }

  /**
   *  Reloads all classes in the package.
   */
  public void reloadPackage() throws Exception {
    ReloadPackageController pkgCtl = new ReloadPackageController(TEST_PACKAGE);
    Reloader  reloader = new Reloader( pkgCtl );
    invokePrint( reloader, TEST_CLASS );
  }

  /**
   * Reloads specified classes using <code>ClassCollectionController</code>.
   */
  public void reloadSelectedClasses() throws Exception {
    ClassCollectionController classCtl = new ClassCollectionController( TEST_CLASS );
    classCtl.addClass(TEST_CLASS2);
    Reloader  reloader = new Reloader( classCtl );
    invokePrint( reloader, TEST_CLASS );
  }

  /**
   * Reloads classes other than specified using <code>ExceptionCollectionController</code>.
   * <br>Interface class should not be reloaded since it is also used
   * by <code>invokePrint</code> method, so interface class should be
   * passed to the <code>ExceptionCollectionController</code>.
   */
  public void reloadExceptionClasses() throws Exception {
    ExceptionCollectionController exCtl = new ExceptionCollectionController( TEST_IFACE );
    Reloader  reloader = new Reloader( exCtl );
    invokePrint( reloader, TEST_CLASS );
  }

  /**
   * Reloads all classes located in the specified directory.
   * <p>In order to test, choose the directory not on the JVM classpath and run the
   * test. Recompile the class and copy it to the directory. New class will be loaded.
   * <p><code>Reloader</code> first looks for the class in the directory, but, if the class
   * is not found there, the parent loader (usually, the system loader) will try to
   * load the class from the JVM classpath. Once it happens, it puts it in cache, so
   * the cached version is always used.
   * <br>Bottom line: do not compile into the JVM classpath, and if you do,
   * move the class out to the different directory.
   *
   */
  public void reloadCustomPath() throws Exception {
    URL url = new URL( "file", null, customPath );
    URL[] urls = { url };
    Reloader reloader = new Reloader( urls, new ReloadAllController() );
    invokePrint( reloader, TEST_CLASS_CUSTOM_PATH );
  }

  /**
   *  Helper method. Calls reloader to load class, instantiates it and
   *  calls print. Used by all <code>reload</code> methods of this class.
   *  @param reloader Properly intialized instance of the reloader class (Controller should be set).
   *  @param testClass Name of the class that will be reloaded. Depending on
   *  <code>Reloader</code> settings, classes used by <code>testClass</code> can also
   *  be reloaded.
   */
  private void invokePrint( Reloader reloader, String testClass ) throws Exception {
    TestIface tst = (TestIface) reloader.reload( testClass ).newInstance();
    tst.print();
  }


  /**
   *  Helper method to read the key.
   *  @return true if repeat key ("R") is pressed.
   */
  private boolean readCommand() throws IOException {
    System.out.println("Change the \"println\" text and recompile to test reloading.");
    System.out.println("Press R to repeat the test and reload, Q to quit, <Enter> to proceed to the next test:");
    BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
    String cmd = br.readLine();
    if (cmd.equalsIgnoreCase("Q"))
      System.exit(0);
    else if (cmd.equalsIgnoreCase("R"))
      return true;

    return false;
  }


  /**
   * Main.
   * Arguments: name of the custom directory (custom path) used for testing
   * custom path reloading. The default is used if not specified.
   */
  public static void main( String[] args ) {
    try {
      if (args.length > 0)
        customPath =  args[0];
      new ReloaderSamples().run();

    }
    catch (Exception ex) {
      System.out.println("!!! Exception:");
      ex.printStackTrace ();
    }

  }
}