// Reloader utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.

package com.myarch.reloader;

import java.util.Collection;
import java.util.ArrayList;


/**
 * Allows for loading the classes listed with <code>ClassCollectionController</code>.
 * Only the specified classes will be re-loaded.
 *
 * @author Alexander Ananiev
 */
public class ClassCollectionController  implements LoadController {

  private Collection classes;

  // ** constructors

  /**
   * Constructs a new <code>ClassCollectionController</code> with the empty
   * class list.
   *
   */
  public ClassCollectionController() {
    // create empty collection to avoid null error
    classes = new ArrayList();
  }

  /**
   * Constructs a new <code>ClassCollectionController</code> with the
   * specified collection of classes.
   * @param classes collection of reloadable classes
   */
  public ClassCollectionController( Collection classes ) {
    setClasses( classes );
  }

  /**
   * Constructs a new <code>ClassCollectionController</code> with the class list
   * containing a single class.
   * @param pClass class name
   */
  public ClassCollectionController( String pClass ) {
    classes = new ArrayList();
    addClass( pClass );
  }

  // ** getters/setters
  
  /**
   * Sets the collection of classes that can be reloaded.
   * @param classes collection of reloadable classes
   */
  public void setClasses( Collection classes ) {
    this.classes =  classes;
  }

  /**
   * Returns the reloadable class list.
   * @return collection of reloadable classes
   */
  public Collection getClasses() {
    return classes;
  }

  /**
   * Adds a reloadable class to the class list.
   * @param pClass class name
   */
  public void addClass( String pClass ) {
    classes.add( pClass );
  }

  // ** search
  
  /**
   * Returns true if class is found in the list of reloadable classes.
   * @param pClass class to look for
   * @return <code>true</code> if a class belongs to the list
   */
  protected boolean isClassFound( String pClass ) {
    return CollectionUtil.isStringElementFound( classes, pClass );
  }



  // ** interface methods

  /**
   * Returns <code>true</code> if a class belongs to the list of
   * reloadable classes and can be reloaded.
   *
   * @param className class name to load
   * @param mainClassName name of the "main" class.
   * Main class is a class passed as a parameter to <code>Reloader.reload()</code> method.
   * @param loader the actual class loader used to load a class.
   * @return <code>true</code> if class <code>className</code> is allowed to be loaded.
   */
  public boolean isReloadAllowed( String className, String mainClassName,
    ClassLoader loader) {

    return isClassFound( className );
  }

  /**
   * Prohibits using parent loader for loadable classes.
   * This ensures that the class is either loaded
   * by the <code>Reloader</code> or  <code>ClassNotFoundException</code> is thrown.
   *
   *
   * @param className class name to load
   * @param mainClassName name of the "main" class.
   * Main class is a class passed as a parameter to <code>Reloader.reload()</code> method.
   * @param loader the actual class loader used to load a class (parent loader).
   * @return <code>false</code> all the time to prohibit using the parent loader.
   */
  public boolean isLoadByParentAllowed( String className, String mainClassName,
    ClassLoader loader) {

    // enforces that the class either reloaded or no found.
    return false;
  }

}