// Reloader utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.

package com.myarch.reloader;

import java.util.Collection;

/**
 * Allows for loading only the classes NOT listed with <code>ExceptionCollectionController</code>.
 * In other words, all classes that can't be reloaded should be specified.
 * You don't need to explicitly list <code>java.lang</code> classes, such as
 * <code>java.lang.Object</code>. These classes throw <code>SecurityException</code>
 * on the attempt to reload, so <code>Reloader</code> immediately passes them on to
 * the parent loader.
 * <p>If you choose not to specify  <code>java.lang</code> classes, default
 * JVM library (${java_home}/jre/lib/rt.jar) should be explicitly added to the classpath.
 *
 * @author Alexander Ananiev
 */
public class ExceptionCollectionController  extends ClassCollectionController {
  // ** constructors

  /**
   * Constructs a new <code>ExceptionCollectionController</code> with the empty
   * class list.
   *
   */
  public ExceptionCollectionController() {
    super();
  }

  /**
   * Constructs a new <code>ExceptionCollectionController</code> with the
   * specified collection of classes.
   * @param classes collection of classes that can't be re-loaded
   */
  public ExceptionCollectionController( Collection classes ) {
    super( classes );
  }

  /**
   * Constructs a new <code>ExceptionCollectionController</code> with a class list
   * containing a single class.
   * @param pClass class name that can't be re-loaded
   */
  public ExceptionCollectionController( String pClass ) {
    super( pClass );
  }

  // ** interface methods

  /**
   * Returns <code>true</code> if a class is NOT among the classes in the class list
   *
   * @param className class name to load
   * @param mainClassName name of the "main" class.
   * Main class is a class passed as a parameter to <code>Reloader.reload()</code> method.
   * @param loader the actual class loader used to load a class.
   * @return <code>true</code> if class <code>className</code> is allowed to be loaded.
   */
  public boolean isReloadAllowed( String className, String mainClassName,
    ClassLoader loader) {

    return ! isClassFound( className );
  }

  /**
   * Prohibits using parent loader for loadable classes.
   * This ensures that the class is either loaded
   * by the <code>Reloader</code> or  <code>ClassNotFoundException</code> is thrown.
   *
   *
   * @param className class name to load
   * @param mainClassName name of the "main" class.
   * Main class is a class passed as a parameter to <code>Reloader.reload()</code> method.
   * @param loader the actual class loader used to load a class (parent loader).
   * @return <code>false</code> all the time to prohibit using the parent loader.
   */
  public boolean isLoadByParentAllowed( String className, String mainClassName,
    ClassLoader loader) {

    // enforces that the class either reloaded or no found.
    return false;
  }

}