// Reloader utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.

package com.myarch.reloader;

/**
 * After <code>Reloader.reload()</code> was called and the main class was loaded, JVM uses the main class' class loader
 * for all classes that the main class uses (calls, extends or refers to). Therefore, it is
 * important to be able to load these classes selectively to avoid <code>ClassCastException</code>.
 * <p><code>Reloader</code> always uses an instance of <code>LoadController</code>
 * to figure out what classes to load.
 * <br><code>Reloader</code> or, more precisely, the class loader created by <code>Reloader</code>,
 * calls <code>LoadController</code> every time it needs to load a class. It only loads classes for which
 * <code>isReloadAllowed</code> returns true.
 *
 * @author Alexander Ananiev
 */

public interface LoadController {

  /**
   *  Class loader will try to load a class only if this method returns <code>true</code>.
   *
   * @param className class name to load
   * @param mainClassName name of the "main" class.
   * Main class is a class passed as a parameter to <code>Reloader.reload()</code> method.
   * @param loader the actual class loader used to load a class.
   * @return <code>true</code> if class <code>className</code> is allowed to be loaded.
   */
  public boolean isReloadAllowed( String className, String mainClassName,
    ClassLoader loader);

  /**
   * If the class loader used by <code>Reloader</code> fails to load a loadable class
   * (<code>isReloadAllowed(...)==true<code>)
   * it may delegate to its parent loader.
   * Most controllers should not allow for using parent loader. Parent loader (which is usually system loader)
   * will cache class files so
   * there is no guarantee that the class will actually be reloaded.
   * Note that this method is called only if <code>isReloadAllowed()</code> previously
   * returned <code>true</code> for this class. For all other classes (non-reloadable classes)
   * parent loader will be called regardless.
   * <br>Returning <code>false</code> from this method ensures that class either reloaded or <code>
   * ClassNotFoundException</code> is thrown.
   *
   *
   * @param className class name to load
   * @param mainClassName name of the "main" class.
   * Main class is a class passed as a parameter to <code>Reloader.reload()</code> method.
   * @param loader the actual class loader used to load a class (parent loader).
   * @return <code>true</code> if class <code>className</code> is allowed to be loaded by the parent loader.
   */
  public boolean isLoadByParentAllowed( String className, String mainClassName,
    ClassLoader loader);


}