// Reloader utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.

package com.myarch.reloader;

import java.net.URL;
import java.net.URLClassLoader;

/**
 * Changes the behaviour of the standard class loader in that it first
 * tries to load the class and only after that calls parent loader.
 * It allows for reloading from the JVM class path.
 * Object of this class must be instantiated before call to <code>reload</code>.
 *
 * @author Alexander Ananiev
 */
class ReloadClassLoader extends URLClassLoader implements Actions {

  private LoadController ctl;
  private LoadActionListener actionListener;


  private String mainClassName;


  /**
   *  All parameters are mandatory. <code>Reloader</code> takes care of the proper defaults.
   */
  ReloadClassLoader( URL[] urls, ClassLoader parent, LoadController ctl,
    LoadActionListener actionListener ) {

    super( urls, parent);
    this.ctl = ctl;
    this.actionListener = actionListener;

  }






  /**
   *  Sets the the class name to reload and
   *  calls <code>loadClass</code> to load the class. Overriden <code>loadClass</code>
   *  ensures that the class will be reloaded.
   *
   * @param name Name of the class to reload, so called "MainClass".
   *
   */
  Class reload( String name ) throws ClassNotFoundException {

    mainClassName = name;

    return loadClass( name );
  }

  /**
   * Reloads classes for which <code>Reloadctl</code> returns <code>true</code>
   * Non-reloadable classes are passed to the parent class loader.
   * Parent class loader must be set by the constructor.
   * Parent class loader should use the same urls as this class loader.
   */
  protected synchronized Class loadClass(String name, boolean resolve)
    throws ClassNotFoundException {

    Class c = null;

    // First, check if the class is allowed to be reloaded.
    // By default main class is always reloadable
    if ( ctl.isReloadAllowed( name, mainClassName, this ) ) {
      try {
        // force reload.
        // causes ClassNotfound or linkException if this loader was not re-created
        actionListener.actionPerformed( BEFORE_RELOAD, name, mainClassName, this );

        c = findClass(name);
        actionListener.actionPerformed( AFTER_RELOAD, name, mainClassName, this );
      }
      catch ( ClassNotFoundException ex) {
        if (ctl.isLoadByParentAllowed( name, mainClassName, getParent() ) )
          c = loadByParent( name );
        else
          throw ex;
      }
      // security exception can be caused by the attempt to load java.lang classes
      catch ( SecurityException ex) {
        // always loaded by parent
        c = loadByParent( name );
      }
    } else
      c = loadByParent( name );

    if (resolve) {
      resolveClass(c);
    }
    return c;

  } // end of loadClass


  private Class loadByParent( String name ) throws ClassNotFoundException  {
    // non-reloadable class -- pass up to parent
    // this will execute normal reloading sequence
    // assume the parent never null -- taken care in constructor
    actionListener.actionPerformed( BEFORE_PARENT_LOAD, name, mainClassName, getParent() );
    Class c = getParent().loadClass(name);
    actionListener.actionPerformed( AFTER_PARENT_LOAD, name, mainClassName, getParent() );

    return c;
  }



} // end of ReloadClassLoader