// Reloader utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.

package com.myarch.reloader;

import java.io.PrintStream;


/**
 * Logs AFTER_RELOAD event produced by <code>Reloader</code> after it successfully loads a class.
 * By default the logging is off. To turn it on, set the following property:
 * <code>reloader.log=yes</code> or <code>java -Dreloader.log=yes</code>
 * You can also use the static methods <code>ReloadLogger.setLogginOn()</code>.
 * @author Alexander Ananiev
 */
public class ReloadLogger implements LoadActionListener, Actions {

  public final static String LOGGING_ENABLE_PROPERTY = "reloader.log";
  private static boolean loggingEnabled = false;

  static {
    String prop = System.getProperty( LOGGING_ENABLE_PROPERTY );
    loggingEnabled = ( prop!=null && prop.equalsIgnoreCase("yes") );
  }

 /**
  * Turns the logging on.
  */
  public static synchronized void setLoggingOn() {
    loggingEnabled = true;
  }

 /**
  * Turns the logging off.
  */
  public static synchronized void setLoggingOff() {
    loggingEnabled = false;
  }

 /**
  * Returns <code>true</code> if the logging is on.
  * @return <code>true</code> if the logging is on
  */
  public static boolean loggingEnabled() {
    return loggingEnabled;
  }

  private PrintStream out = System.out;
  private String prefix = "[Reloader]  ";


  /**
   * Construct a new <code>ReloadLogger</code> wich will use <code>System.out</code>
   * for logging.
   */
  public ReloadLogger() {
  }

  /**
   * Construct a new <code>ReloadLogger</code> for the specified <code>PrintStream</code>.
   * @param out output stream to use for logging
   */
  public ReloadLogger( PrintStream out ) {
    setOutStream( out );
  }

  /**
   * Sets the output stream for logging.
   * @param out stream to use for logging
   */
  public void setOutStream( PrintStream out){
    this.out = out;
  }


  /**
   * Sets the prefix for all logger messages. The default is ""[Reloader]  ".
   * @param prefix prefix for messages
   */
  public void setMessagePrefix( String prefix ){
    this.prefix = prefix;
  }
  /**
   * Returns current prefix for logger messages.
   * @return prefix
   */
  public String getMessagePrefix() {
    return prefix;
  }

 /**
  * <code>Reloader</code> calls this method every time it tries to load a class.
  * <code>ReloadLogger</code> only logs AFTER_RELOAD and ignores other events.
  *
  * @param action type of a loading event, such as "before loading by reloader", "successful loading by reloader",
  * "successful loading by parent loader" and so on. See <code>Actions</code> class for details.
  * @param name name of a class being loaded.
  * @param mainClassName name of the "main" class, i.e., the class passed as a parameter to <code>reload</code> method.
  * @param loader the actual class loader used to load a class. Can be a class loader created by <code>Reloader</code>
  * or a parent class loader, such as system class loader.
  */
  public void actionPerformed( int action, String name, String mainClassName,
    ClassLoader loader ) {

    if ( loggingEnabled() && action == AFTER_RELOAD )
      out.println( prefix+"Reloaded class: "+ name);
  }

}