// Reloader utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.

package com.myarch.reloader;

import java.util.Collection;
import java.util.ArrayList;


/**
 * Allows loading all classes in the packages listed with <code>ReloadPackageController</code>.
 *
 * @author Alexander Ananiev
 */
public class ReloadPackageController  implements LoadController {

  private Collection packages;

  // ** constructors

  /**
   * Constructs a new <code>ReloadPackageController</code> with the empty
   * package list.
   *
   */
  public ReloadPackageController() {
    // create empty collection to avoid null error
    packages = new ArrayList();
  }

  /**
   * Constructs a new <code>ReloadPackageController</code> with the
   * specified collection of packages.
   * @param packages collection of packages. All classes in these packages can be
   * reloaded.
   */
  public ReloadPackageController( Collection packages ) {
    setPackages( packages );
  }

  /**
   * Constructs a new <code>ReloadPackageController</code> with a package list
   * containing a single package.
   * @param pPackage package with reloadable classes
   */
  public ReloadPackageController( String pPackage ) {
    packages = new ArrayList();
    addPackage( pPackage );
  }

  // ** getters/setters

  /**
   * Sets the collection of packages.
   * @param packages collection of packages. All classes in these packages can be
   * reloaded.
   */
  public void setPackages( Collection packages ) {
    this.packages =  packages;
  }

  /**
   * Returns the current list (collection) of packages.
   * @return package list
   */
  public Collection getPackages() {
    return packages;
  }


  /**
   * Add a package to the list of packages with reloadable classes.
   * @param pPackage package with reloadable classes
   */
  public void addPackage( String pPackage ) {
    packages.add( pPackage );
  }

  // ** search

  private boolean isPackageFound( String pPackage ) {
    return CollectionUtil.isStringElementFound( packages, pPackage );
  }


  // ** interface methods

  /**
   * Returns <code>true</code> if a class belongs to one of the packages from the
   * package list.
   *
   * @param className class name to load
   * @param mainClassName name of the "main" class.
   * Main class is a class passed as a parameter to <code>Reloader.reload()</code> method.
   * @param loader the actual class loader used to load a class.
   * @return <code>true</code> if class <code>className</code> is allowed to be loaded.
   */
  public boolean isReloadAllowed( String className, String mainClassName,
    ClassLoader loader) {

    String pkg = ClassNameUtil.getPackageName( className );

    return isPackageFound( pkg );

  }

  /**
   * Prohibits using parent loader for loadable classes.
   * This ensures that the class is either loaded
   * by the <code>Reloader</code> or  <code>ClassNotFoundException</code> is thrown.
   *
   *
   * @param className class name to load
   * @param mainClassName name of the "main" class.
   * Main class is a class passed as a parameter to <code>Reloader.reload()</code> method.
   * @param loader the actual class loader used to load a class (parent loader).
   * @return <code>false</code> all the time to prohibit using the parent loader.
   */
  public boolean isLoadByParentAllowed( String className, String mainClassName,
    ClassLoader loader) {

    // enforces that the class either reloaded or no found.
    return false;
  }

}