// Reloader utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.


package com.myarch.reloader;

import java.net.URL;
import java.net.URLClassLoader;

/**
 * Performs dynamic class loading (reloading).
 * <p>Uses controller object to figure out what classes to load.
 * <br>When <code>reload()</code> is called, loads the class passed as the
 * parameter and, depending on the controller, called classes.
 * <p>For every <code>reload()</code> call creates the new class loader
 * to clear the cache of loaded classes.
 *
 * @author Alexander Ananiev
 */
public class Reloader {

  // static section to control reloading
  private static boolean reloadEnabled = true;

  /**
   * System property to turn reloading on/off.
   * <br>On: <code>-Dreloader.reload=yes</code> (or any value other than "no")
   * <br>Off: <code>-Dreloader.reload=no</code>
   * <br>On is the default.
   */
  public final static String RELOAD_ENABLE_PROPERTY = "reloader.reload";

  static {
    String prop = System.getProperty( RELOAD_ENABLE_PROPERTY );
    reloadEnabled = ( prop == null || ! prop.equalsIgnoreCase("no") );
  }

  /**
   * Turns reloading on for all instances of <code>Reloader</code>.
   * <br>On is the default, so use it after <code>setReloadOff()</code>.
   */
  public static synchronized void setReloadOn() {
    reloadEnabled = true;
  }

  /**
   * Turns reloading off for all instances of <code>Reloader</code>.
   * <br>Same with setting <code>-Dreloader.reload=no</code>.
   * <br>Do it for production system, when there is no need to frequently update
   * the code.
   */
  public static synchronized void setReloadOff() {
    reloadEnabled = false;
  }

  /**
   * @return true if reloading is on
   */
  public static boolean reloadEnabled() {
    return reloadEnabled;
  }


  // private attributes
/*
*  private URL[] urls = ((URLClassLoader) ClassLoader.getSystemClassLoader()).getURLs();
*  private ClassLoader parent = ClassLoader.getSystemClassLoader();
*  private LoadController ctl = new ReloadMainController();
*  private LoadActionListener actionListener = new ReloadLogger();
*/
  private URL[] urls;
  private ClassLoader parent;
  private LoadController ctl;
  private LoadActionListener actionListener;


  // * constructor section

  /**
   * Constructs a new <code>Reloader</code> with default settings.
   * <code>ReloadMainController</code> is the default, so only main class
   * (the class passed to <code>reload()</code> as a parameter) will be reloaded.
   */
  public Reloader( ) {
    init();
  }


  /**
   * Constructs a new <code>Reloader</code> for the specified URLs and
   * default controller.
   * <br><code>Reloader</code> uses class loader inherited from <code>URLClassLoader</code>,
   * so URLs are passed to the <code>URLClassLoader</code> during <code>reload()</code> call.
   * <br>Set URLs only if classes should be loaded from the custom class path outside of
   * JVM class path.
   *
   * @param urls the URLs from which to load classes
   */
  public Reloader( URL[] urls ) {
    init();
    setURLs( urls );
  }

  /**
   * Constructs a new <code>Reloader</code> for the specified URLs, parent class loader  and
   * default controller.
   * <br>Set URLs only if classes should be loaded from the custom class path outside of
   * JVM class path.
   * <br>System class loader (<code>ClassLoader.getSystemClassLoader()</code>) is used
   * as the default parent.
   *
   * @param urls the URLs from which to load classes
   * @param parent the parent class loader for delegation
   */
  public Reloader( URL[] urls, ClassLoader parent ) {
    init();
    setParent( parent );
    setURLs( urls );
  }

  /**
   * Constructs a new <code>Reloader</code> for the specified controller.
   * @param ctl controller object. Controller provides <code>Reloader</code> with the list
   * of classes for loading.
   */
  public Reloader( LoadController ctl ) {
    init();
    setLoadController( ctl );
  }

  /**
   * Constructs a new <code>Reloader</code> for the specified URLs and controller.
   * <br>Set URLs only if classes should be loaded from the custom class path outside of
   * JVM class path.
   *
   * @param urls the URLs from which to load classes
   * @param ctl controller object. Controller provides <code>Reloader</code> with the list
   * of classes for loading.
   *
   */
  public Reloader( URL[] urls, LoadController ctl ) {
    init();
    setURLs( urls );
    setLoadController( ctl );
  }

  /**
   * Constructs a new <code>Reloader</code> and sets
   * parent class loader  for <code>URLClassLoader</code> that will be
   * used for <code>reload()</code> call.
   * Normally, it should be the loader of the calling class, i.e.
   * <br><code>reloader = new Reloader( this.getClass().getClassLoader() );</code>
   * <br>URLs (loading classpath) will be taken from this parent loader if they are not
   * specified using <code>setURLs()</code>.
   * @param parent the parent class loader for delegation
   */
  public Reloader(  ClassLoader parent ) {
    setParent( parent );
    setURLs( getDefaultURLs() );
    setLoadController( getDefaultLoadController() );
    setLogger( getDefaultActionListener() );

  }


  private void init() {
    setParent( getDefaultParent() );
    setURLs( getDefaultURLs() );
    setLoadController( getDefaultLoadController() );
    setLogger( getDefaultActionListener() );
  }

  // * end of constructor section

  /**
   * Sets the controller object for <code>Reloader</code>. Controller provides
   * <code>Reloader</code> with the list of classes for loading.
   *
   * @param ctl controller object.
   *
   */
  public void setLoadController( LoadController ctl ) {
    this.ctl = ctl;
  }

  /**
   * Sets URLs for <code>Reloader</code>.
   * <br><code>Reloader</code> uses class loader inherited from <code>URLClassLoader</code>,
   * so URLs are passed to the <code>URLClassLoader</code> during <code>reload()</code> call.
   * <br>Set URLs only if classes should be loaded from the custom class path outside of
   * JVM class path.
   *
   * @param urls the URLs from which to load classes
   */
  public void setURLs( URL[] urls ) {
    this.urls = urls;
  }

  /**
   * Returns the current array of URLs for class loader.
   *
   * @return urls the URLs from which to load classes
   */
  public URL[] getURLs() {
    return urls;
  }


  /**
   * Sets parent class loader  for <code>URLClassLoader</code> that will be
   * used for <code>reload()</code> call.
   * Normally, it should be the loader of the calling class, i.e.
   * <br><code>reloader = new Reloader( this.getClass().getClassLoader() );</code>
   *
   * @param parent the parent class loader for delegation
   */
  public void setParent( ClassLoader parent ) {
    this.parent = parent;
  }

  /**
   * Returns currently set parent class loader.
   *
   * @param parent the parent class loader for delegation
   */
  public ClassLoader getParent() {
    return parent;
  }


  /**
   * Sets custom logger. The default logger is <code>ReloadLogger</code>.
   * To create custom logger, implement <code>LoadActionListener</code> interface.
   *
   * @param logger custom logger
   */

  public void setLogger( LoadActionListener logger ){
    actionListener = logger;
  }


  // defaults

  private URL[] getDefaultURLs() {

    //return ((URLClassLoader) ClassLoader.getSystemClassLoader()).getURLs();
    return ((URLClassLoader) parent).getURLs();

  }


  private ClassLoader getDefaultParent() {

    return ClassLoader.getSystemClassLoader();
  }

  // checker
  private LoadController getDefaultLoadController( ) {
    return new ReloadMainController();
  }

  // action listener
  private LoadActionListener getDefaultActionListener() {
    return new ReloadLogger();
  }

  /**
   * Loads the class and classes used by this class depending on the controller.
   * <br>Delegates loading to the new class loader (subclass of <code>URLClassLoader</code>).
   * For each class that should be loaded, the class loader calls the controller.
   * If the controller permits to load the class, the class loader tries to load it.
   * If it can not load the class,
   * it calls its parent class loader.
   *
   * @param name class name to load
   * @throws ClassNotFoundException if class loader or its parent did not find the class
   *
   */
  public Class reload( String name ) throws ClassNotFoundException {

    // resulted class
    Class c = null;
    if ( reloadEnabled() ) {
      // create new loader in order to avoid "class is already loaded" error.
      //##debug
      System.out.println("--- Reload urls:");
      for(int i=0; i<urls.length; ++i)
        System.out.println("\n"+urls[i]);

      ReloadClassLoader reloadldr  = new ReloadClassLoader( urls, parent,
        ctl, actionListener );
      c = reloadldr.reload( name );
    }
    else {
      ClassLoader classldr = parent;
      c =  classldr.loadClass( name );
    }

    return c;
  }
}