// Reloader utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.


import java.net.URLClassLoader;
import java.net.URL;
import com.myarch.reloader.*;

import junit.framework.*;



/**
 * Reloader test harness.
 * Junit (www.junit.org) framework is used to run the tests.
 *
 * @author Alexander Ananiev
 */

public class Tester extends TestCase implements TestConstants {


  private  static String customPath = CUSTOM_PATH;
  private  static String testClassCustomPath = TEST_CLASS_CUSTOM_PATH;
  private  static String testIface = TEST_IFACE;
  private  static String testPackage = TEST_PACKAGE;
  private  static  String testClass = TEST_CLASS;

  public Tester( String name ) {
    super( name );
  }

  protected void setUp() throws Exception {
    ReloadLogger.setLoggingOn();
  }

  /**
   * Test case 1
   * Reload from one of the directories on the system classpath.
   * use default reloader (ReloadMain).
   *
   */
  public void testReloadSystemPath() throws Exception {
    Reloader reloader = new Reloader();
    runReloadTest("System path", reloader, testClass, testIface );
  }

  /**
   * Test case 2
   * Package -based reloading.
   */
  public void testReloadPackage() throws Exception {
    Reloader reloader = new Reloader();
    ReloadPackageController pkgCtl = new ReloadPackageController(testPackage);
    reloader.setLoadController( pkgCtl );
    runReloadTest("Package", reloader, testClass, testIface );
  }

  /**
   * Test case 3
   * Reload only the specified classes.
   */
  public void testReloadSelectedClasses() throws Exception {
    Reloader reloader = new Reloader();
    ClassCollectionController classCtl = new ClassCollectionController( testClass);
    reloader.setLoadController( classCtl );
    runReloadTest("ClassCollection", reloader, testClass, testIface );
  }

  /**
   *  Test case 4
   *  Reload all except the specified classes.
   *
   */
  public void testReloadExceptionClasses() throws Exception {
    Reloader reloader = new Reloader();
    ExceptionCollectionController exCtl = new ExceptionCollectionController( testIface );
    //exCtl.addClass("java.lang.Object");
    reloader.setLoadController( exCtl );
    runReloadTest("ExceptionCollection", reloader, testClass, testIface );

  }

  /**
   * Test case 5
   * Reloading from the custom path with test class.
   * All classes are reloadable in this dir (we check only one).
   */
  public void testReloadCustomPath() throws Exception {
    Reloader reloader = new Reloader();
    reloader.setLoadController( new ReloadAllController() );
    URL url = new URL( "file", null, customPath );
    URL[] urls = { url };
    reloader.setURLs( urls );
    runReloadTest("Custom path", reloader, testClassCustomPath, testIface );
  }

  /**
   *  Tests the reloan on/reload off.
   */

  public void testReloadOn() throws Exception  {
    Reloader reloader = new Reloader();
    ClassLoader systemldr = ClassLoader.getSystemClassLoader();
    Class c = null;
    Reloader.setReloadOff();
    c = reloader.reload( testClass );
    assert("Class is loaded by system loader",
      (c.getClassLoader() == systemldr) );
    Reloader.setReloadOn();
    c = reloader.reload( testClass );
    assert("Class is loaded by non-system loader",
      (c.getClassLoader() != systemldr) );

  }


  /**
   * Reloads the class and verifies that it was reloaded by
   * comparing classloaders.
   */
  private void runReloadTest( String testName, Reloader reloader, String className,
    String ifaceName ) throws Exception {

    ClassLoader systemldr = ClassLoader.getSystemClassLoader();
    Class c = null;

    c = reloader.reload( className );
    assert(testName+":"+"Class is loaded by non-system loader",
      (c.getClassLoader() != systemldr) );


    Class c1 = reloader.reload( className );
    assert(testName+":"+"Class is always loaded by the new loader",
      c.getClassLoader() != c1.getClassLoader()  );

    // this will cause the loading of TestIface
    Class ifaceClass = Class.forName( ifaceName );

    // make sure that interface is not reloaded and it can be used for casting
    assert(testName+":"+"Interface implemented by the class is loaded by system loader",
      ifaceClass.isAssignableFrom( c )  );
 }

  /**
   * Main.
   * Arguments: name of the custom directory (custom path) used for testing
   * custom path reloading. The default is used if not specified.
   */
  public static void main( String[] args ) {
    try {
      if (args.length > 0)
        customPath =  args[0];

      junit.textui.TestRunner.run( Tester.class );

    }
    catch (Exception ex) {
      System.out.println("!!! Exception:");
      ex.printStackTrace ();
    }
  }
}